// Header File
// Created 03/11/2005; 17:53:01
#ifndef __GTC__
asm("utsrlib__version01: .xdef utsrlib__version01");
#else
char utsrlib__version01[0];
#endif

#define utsrlib__utsr_install utsrlib__0000
short utsrlib__utsr_install(const void *base_address, long int size, short functions_number, const void* functions_list, short flags);
/*
; base_address : beginning of the code of the TSR to install
;    this code must begin by the name (exactly 8 chars) followed by a blank short (which will be filled by the UtsrId)
; 
; size : total size of data to copy in RAM
;
; functions_number : number of UTSR functions
;
; functions_list : pointer to the functions list, which must have exactly functions_number elements
;
; flags : flags which set utsr_install behaviour 
;
;
; returns UTSR_NULL in case of error, utsrId else (utsrId is the handle in which the utsr is copied).
;
*/



#define utsr_delete  utsrlib__0001
void utsr_delete(short utsrId, short flags);
/*
; utsrId : utsrId of the utsr to remove from the list
;
; flags : flags wich set utsr_delete behaviour
;
*/

#define utsr_find utsrlib__0002
short utsr_find(const char* utsrName);
/*
;utsrName : pointer to an array of 8 chars, which contains the name of the searched utsr
; 
; 
; returns UTSR_NULL in case of error (utsr not found, by example), utsrId else.
;
*/


#define utsr_get_handle utsrlib__0003
HANDLE utsr_get_handle(void);
/*
;
; returns the handle of utsr list, or H_NULL in case of error 
;
*/

#define utsrlib__kernel_uninstall utsrlib__0004
void utsrlib__kernel_uninstall(void);
/*
;
; warns all utsrs from a kernel uninstallation
; removes all utsrs from the utsr list
; keep in mind that even in case of errors, all utsrs will be removed (as wanted by PpHd ;) )
;
*/



/*
Flags used in utsr_install and utsr_delete functions
*/

 
#define UTSR_AUTHORIZE_TWINS 1 /* authorizes multiples instances of the same utsr*/
#define UTSR_AUTHORIZE_SL 2 /* authorizes messages in the status line*/
#define UTSR_AUTHORIZE_DLG 4 /* authorizes messages through dialog boxes*/
#define UTSR_ERRORS_ONLY 8 /* only warnings will be displayed*/

#define UTSR_AUTHORIZE_TWINS_BIT 0
#define UTSR_AUTHORIZE_SL_BIT 1
#define UTSR_AUTHORIZE_DLG_BIT 2
#define UTSR_ERRORS_ONLY_BIT 3


#define UTSR_CAN_DELETE 0x7FFF
#define UTSR_DELETE 0x7FFE
#define UTSR_ICON 0x7FFD
#define UTSR_ABOUT 0x7FFC
#define UTSR_INSTALL 0x7FFB
#define UTSR_FORMAT 0x7FFA
#define UTSR_HELP 0x7FF9
#define UTSR_RELOAD 0x7FF8
#define UTSR_KERNEL_UNINSTALL 0x7FF7
#define UTSR_CMD_00 0x7F00
#define UTSR_CMD_01 0x7F01
#define UTSR_CMD_02 0x7F02
#define UTSR_CMD_03 0x7F03
#define UTSR_CMD_04 0x7F04
#define UTSR_CMD_05 0x7F05
#define UTSR_CMD_06 0x7F06
#define UTSR_CMD_07 0x7F07
#define UTSR_CMD_08 0x7F08
#define UTSR_CMD_09 0x7F09
#define UTSR_CMD_0A 0x7F0A
#define UTSR_CMD_0B 0x7F0B
#define UTSR_CMD_0C 0x7F0C
#define UTSR_CMD_0D 0x7F0D
#define UTSR_CMD_0E 0x7F0E
#define UTSR_CMD_0F 0x7F0F



#define UTSR_NULL 0x0000
#define UTSR_AUTOINT_1 0x0601
#define UTSR_AUTOINT_2 0x0602
#define UTSR_AUTOINT_3 0x0603
#define UTSR_AUTOINT_4 0x0604
#define UTSR_AUTOINT_5 0x0605
#define UTSR_AUTOINT_6 0x0606
#define UTSR_AUTOINT_7 0x0607
#define UTSR_TRAP_0 0x0610
#define UTSR_TRAP_1 0x0611
#define UTSR_TRAP_2 0x0612
#define UTSR_TRAP_3 0x0613
#define UTSR_TRAP_4 0x0614
#define UTSR_TRAP_5 0x0615
#define UTSR_TRAP_6 0x0616
#define UTSR_TRAP_7 0x0617
#define UTSR_TRAP_8 0x0618
#define UTSR_TRAP_9 0x0619
#define UTSR_TRAP_10 0x061A
#define UTSR_TRAP_11 0x061B
#define UTSR_TRAP_12 0x061C
#define UTSR_TRAP_13 0x061D
#define UTSR_TRAP_14 0x061E
#define UTSR_TRAP_15 0x061F
#define UTSR_EV_HOOK 0x0620
#define UTSR_EMPTY 0x0631
#define UTSR_UNKNOWN 0x0E30
#define UTSR_TSRLIB 0x0E32
#define UTSR_IMPORT 0x0633
#define UTSR_EXPORT 0x0634
#define UTSR_pFRAME 0x0E35


